Module orx_split_vec::prelude
source · Expand description
The split-vec prelude, along with the SplitVec, imports
various growth startegies, iterators and finally the orx_pinned_vec::PinnedVec trait.
Structs
- Stategy which allows creates a fragment with double the capacity of the prior fragment every time the split vector needs to expand.
- Stategy which allows new fragments grow exponentially.
- A contagious fragment of the split vector.
- Iterator over the
SplitVec. - Stategy which allows the split vector to grow linearly.
- A split vector; i.e., a vector of fragments, with the following features:
Enums
- Returns the result of trying to get a slice as a contagious memory from the split vector.
Traits
- Growth strategy of a split vector.
- Marker trait for types to be contained in a
PinnedVecwhich do not hold a field which is a reference to a another element of the same vector. - A vector of elements of T which differs from the
std::vec::Vecby the following feature: PinnedVecSimpleis aPinnedVecwhere the elements satisfy the trait bound `T: NotSelfRefVecItem.- A type to be contained in a
PinnedVecwhich does hold a field which is a reference to a another element of the same vector.