Fragment

Struct Fragment 

pub struct Fragment<T> { /* private fields */ }
A contiguous fragment of the split vector.

Suppose a split vector contains 10 integers from 0 to 9. Depending on the growth strategy of the split vector, this data might be stored in 3 contiguous fragments, say [0, 1, 2, 3], [4, 5, 6, 7] and [8, 9].

Implementations

Source

impl<T> Fragment<T>

pub fn new(capacity: usize, data: Vec<T>) -> Self

Creates a fragment from data with the target logical capacity.

pub fn new_empty(capacity: usize) -> Self

Creates a new fragment with the given capacity.

pub fn into_inner(self) -> Vec<T>

Consumes the fragment and returns the inner vector.

pub fn has_capacity_for_one(&self) -> bool

Returns whether the fragment has room to push a new item or not.

pub fn room(&self) -> usize

Returns the available capacity in the fragment.

pub fn len(&self) -> usize

Returns the number of initialized elements in the fragment.

pub fn is_empty(&self) -> bool

Returns true if the fragment contains no elements.

pub fn capacity(&self) -> usize

Returns the logical capacity of the fragment.

pub fn as_slice(&self) -> &[T]

Returns a shared slice view of initialized elements.

pub fn as_ptr(&self) -> *const T

Returns a raw pointer to the fragment’s initialized elements.

pub fn binary_search_by<F>(&self, f: F) -> Result<usize, usize>
where F: FnMut(&T) -> Ordering,

Binary-searches initialized elements with a comparator.

pub fn iter(&self) -> Iter<'_, T>

Returns an iterator over initialized elements.

pub fn last(&self) -> Option<&T>

Returns the last initialized element, if any.

pub fn first(&self) -> Option<&T>

Returns the first initialized element, if any.

pub fn get(&self, index: usize) -> Option<&T>

Returns a shared reference to the element at index, if present.

pub unsafe fn get_unchecked(&self, index: usize) -> &T

Returns a shared reference to the element at index without bounds checks.

Safety

Caller must ensure index < self.len().

pub fn push(&mut self, value: T)

Appends an element to the end of initialized region.

pub unsafe fn set_len(&mut self, new_len: usize)

Sets the initialized length of the fragment.

Safety

Caller must uphold Vec::set_len invariants.

pub unsafe fn as_mut_vec(&mut self) -> &mut Vec<T>

SAFETY

Obtained reference to the vector can be used to change the length of the vector by adding or removing elements; however, it must not change the capacity and underlying allocation of the vector.

pub fn as_mut_ptr(&mut self) -> *mut T

Returns a mutable raw pointer to initialized elements.

pub fn extend_from_slice(&mut self, slice: &[T])
where T: Clone,

Clones and appends all elements from slice.

pub unsafe fn get_unchecked_mut(&mut self, index: usize) -> &mut T

Returns a mutable reference to the element at index without bounds checks.

Safety

Caller must ensure index < self.len() and aliasing rules are respected.

pub fn pop(&mut self) -> Option<T>

Removes and returns the last element, if any.

pub fn insert(&mut self, index: usize, element: T)

Inserts element at index, shifting later elements to the right.

pub fn remove(&mut self, index: usize) -> T

Removes and returns the element at index, shifting later elements left.

pub fn clear(&mut self)

Removes all initialized elements from the fragment.

pub fn truncate(&mut self, len: usize)

Truncates the initialized length to at most len.

pub fn swap(&mut self, a: usize, b: usize)

Swaps two initialized elements.

pub fn iter_mut(&mut self) -> IterMut<'_, T>

Returns a mutable iterator over initialized elements.

pub fn sort_by<F>(&mut self, compare: F)
where F: FnMut(&T, &T) -> Ordering,

Sorts initialized elements with the given comparator.