Module orx_split_vec::prelude
source · Expand description
The split-vec prelude, along with the SplitVec, imports
various growth startegies, iterators and finally the orx_pinned_vec::PinnedVec trait.
Structs
- Stategy which allows to define a custom growth rate with a function of priorly created fragments.
- Stategy which allows creates a fragment with double the capacity of the prior fragment every time the split vector needs to expand.
- Stategy which allows new fragments grow exponentially.
- A contagious fragment of the split vector.
- Stategy which allows the split vector to grow linearly.
- A split vector; i.e., a vector of fragments, with the following features:
- Iterator over the
SplitVec.
Enums
- Returns the result of trying to get a slice as a contagious memory from the split vector.
Traits
- A vector of elements of T which differs from the
std::vec::Vecby the following feature: - Growth strategy of a split vector.